@@ -64,6 +64,7 @@ import com.sun.jna.Pointer;
 import com.sun.jna.platform.win32.BaseTSD.SIZE_T;
 import com.sun.jna.platform.win32.BaseTSD.ULONG_PTR;
 import com.sun.jna.platform.win32.BaseTSD.ULONG_PTRByReference;
+import com.sun.jna.platform.win32.COM.COMUtils;
 import com.sun.jna.platform.win32.Ntifs.REPARSE_DATA_BUFFER;
 import com.sun.jna.platform.win32.Ntifs.SymbolicLinkReparseBuffer;
 import com.sun.jna.platform.win32.WinBase.FILETIME;
@@ -84,6 +85,7 @@ import com.sun.jna.platform.win32.WinDef.HWND;
 import com.sun.jna.platform.win32.WinDef.USHORT;
 import com.sun.jna.platform.win32.WinNT.HANDLE;
 import com.sun.jna.platform.win32.WinNT.HANDLEByReference;
+import com.sun.jna.platform.win32.WinNT.HRESULT;
 import com.sun.jna.platform.win32.WinNT.MEMORY_BASIC_INFORMATION;
 import com.sun.jna.platform.win32.WinNT.OSVERSIONINFO;
 import com.sun.jna.platform.win32.WinNT.OSVERSIONINFOEX;
@@ -91,6 +93,7 @@ import com.sun.jna.ptr.IntByReference;
 import com.sun.jna.ptr.ShortByReference;
 
 import junit.framework.TestCase;
+import org.junit.Assume;
 
 public class Kernel32Test extends TestCase {
 
@@ -727,42 +730,39 @@ public class Kernel32Test extends TestCase {
             ShortByReference lpBuffer = new ShortByReference();
             IntByReference lpBytes = new IntByReference();
 
-            if (false == Kernel32.INSTANCE.DeviceIoControl(hFile,
+            AbstractWin32TestSupport.assertCallSucceeded("DeviceIoControl",
+                    Kernel32.INSTANCE.DeviceIoControl(hFile,
                     FSCTL_GET_COMPRESSION,
                     null,
                     0,
                     lpBuffer.getPointer(),
                     USHORT.SIZE,
                     lpBytes,
-                    null)) {
-                fail("DeviceIoControl failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+                    null));
             assertEquals(WinNT.COMPRESSION_FORMAT_NONE, lpBuffer.getValue());
             assertEquals(USHORT.SIZE, lpBytes.getValue());
 
             lpBuffer = new ShortByReference((short)WinNT.COMPRESSION_FORMAT_LZNT1);
 
-            if (false == Kernel32.INSTANCE.DeviceIoControl(hFile,
+            AbstractWin32TestSupport.assertCallSucceeded("DeviceIoControl",
+                    Kernel32.INSTANCE.DeviceIoControl(hFile,
                     FSCTL_SET_COMPRESSION,
                     lpBuffer.getPointer(),
                     USHORT.SIZE,
                     null,
                     0,
                     lpBytes,
-                    null)) {
-                fail("DeviceIoControl failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+                    null));
 
-            if (false == Kernel32.INSTANCE.DeviceIoControl(hFile,
+            AbstractWin32TestSupport.assertCallSucceeded("DeviceIoControl",
+                    Kernel32.INSTANCE.DeviceIoControl(hFile,
                     FSCTL_GET_COMPRESSION,
                     null,
                     0,
                     lpBuffer.getPointer(),
                     USHORT.SIZE,
                     lpBytes,
-                    null)) {
-                fail("DeviceIoControl failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+                    null));
             assertEquals(WinNT.COMPRESSION_FORMAT_LZNT1, lpBuffer.getValue());
             assertEquals(USHORT.SIZE, lpBytes.getValue());
 
@@ -795,9 +795,8 @@ public class Kernel32Test extends TestCase {
                     WinNT.FILE_ATTRIBUTE_DIRECTORY | WinNT.FILE_FLAG_BACKUP_SEMANTICS | WinNT.FILE_FLAG_OPEN_REPARSE_POINT,
                     null);
 
-            if (WinBase.INVALID_HANDLE_VALUE.equals(hFile)) {
-                fail("CreateFile failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+            AbstractWin32TestSupport.assertCallSucceeded("CreateFile",
+                    !WinBase.INVALID_HANDLE_VALUE.equals(hFile));
 
             try {
                 SymbolicLinkReparseBuffer symLinkReparseBuffer = new SymbolicLinkReparseBuffer(folder.getFileName().toString(),
@@ -806,7 +805,8 @@ public class Kernel32Test extends TestCase {
 
                 REPARSE_DATA_BUFFER lpBuffer = new REPARSE_DATA_BUFFER(WinNT.IO_REPARSE_TAG_SYMLINK, (short) 0, symLinkReparseBuffer);
 
-                assertTrue(Kernel32.INSTANCE.DeviceIoControl(hFile,
+                AbstractWin32TestSupport.assertCallSucceeded("DeviceIoControl",
+                        Kernel32.INSTANCE.DeviceIoControl(hFile,
                         FSCTL_SET_REPARSE_POINT,
                         lpBuffer.getPointer(),
                         lpBuffer.getSize(),
@@ -817,7 +817,8 @@ public class Kernel32Test extends TestCase {
 
                 Memory p = new Memory(REPARSE_DATA_BUFFER.sizeOf());
                 IntByReference lpBytes = new IntByReference();
-                assertTrue(Kernel32.INSTANCE.DeviceIoControl(hFile,
+                AbstractWin32TestSupport.assertCallSucceeded("DeviceIoControl",
+                        Kernel32.INSTANCE.DeviceIoControl(hFile,
                         FSCTL_GET_REPARSE_POINT,
                         null,
                         0,
@@ -828,7 +829,7 @@ public class Kernel32Test extends TestCase {
                 // Is a reparse point
                 lpBuffer = new REPARSE_DATA_BUFFER(p);
                 assertTrue(lpBytes.getValue() > 0);
-                assertTrue(lpBuffer.ReparseTag == WinNT.IO_REPARSE_TAG_SYMLINK);
+                assertEquals(WinNT.IO_REPARSE_TAG_SYMLINK, lpBuffer.ReparseTag);
                 assertEquals(folder.getFileName().toString(), lpBuffer.u.symLinkReparseBuffer.getPrintName());
                 assertEquals(folder.getFileName().toString(), lpBuffer.u.symLinkReparseBuffer.getSubstituteName());
             } finally {
@@ -1020,44 +1021,41 @@ public class Kernel32Test extends TestCase {
         try {
 
             Memory p = new Memory(FILE_BASIC_INFO.sizeOf());
-            if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileBasicInfo, p, new DWORD(p.size()))) {
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+            AbstractWin32TestSupport.assertCallSucceeded("GetFileInformationByHandleEx for FileBasicInfo (" + p.size() + " bytes)",
+                    Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileBasicInfo, p, new DWORD(p.size())));
             FILE_BASIC_INFO fbi = new FILE_BASIC_INFO(p);
             // New file has non-zero creation time
             assertTrue(0 != fbi.CreationTime.getValue());
 
             p = new Memory(FILE_STANDARD_INFO.sizeOf());
-            if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileStandardInfo, p, new DWORD(p.size()))) {
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+            AbstractWin32TestSupport.assertCallSucceeded("GetFileInformationByHandleEx for FileStandardInfo (" + p.size() + " bytes)",
+                    Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileStandardInfo, p, new DWORD(p.size())));
             FILE_STANDARD_INFO fsi = new FILE_STANDARD_INFO(p);
             // New file has 1 link
             assertEquals(1, fsi.NumberOfLinks);
 
             p = new Memory(FILE_COMPRESSION_INFO.sizeOf());
-            if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileCompressionInfo, p, new DWORD(p.size()))) {
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+            AbstractWin32TestSupport.assertCallSucceeded("GetFileInformationByHandleEx for FileCompressionInfo (" + p.size() + " bytes)",
+                    Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileCompressionInfo, p, new DWORD(p.size())));
             FILE_COMPRESSION_INFO fci = new FILE_COMPRESSION_INFO(p);
             // Uncompressed file should be zero
             assertEquals(0, fci.CompressionFormat);
 
             p = new Memory(FILE_ATTRIBUTE_TAG_INFO.sizeOf());
-            if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileAttributeTagInfo, p, new DWORD(p.size()))) {
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+            AbstractWin32TestSupport.assertCallSucceeded("GetFileInformationByHandleEx for FileAttributeTagInfo (" + p.size() + " bytes)",
+                    Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileAttributeTagInfo, p, new DWORD(p.size())));
             FILE_ATTRIBUTE_TAG_INFO fati = new FILE_ATTRIBUTE_TAG_INFO(p);
             // New files have the archive bit
             assertEquals(WinNT.FILE_ATTRIBUTE_ARCHIVE, fati.FileAttributes & WinNT.FILE_ATTRIBUTE_ARCHIVE);
 
-            p = new Memory(FILE_ID_INFO.sizeOf());
-            if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileIdInfo, p, new DWORD(p.size()))) {
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
+            if (VersionHelpers.IsWindowsServer() && VersionHelpers.IsWindows8OrGreater()) {
+                p = new Memory(FILE_ID_INFO.sizeOf());
+                AbstractWin32TestSupport.assertCallSucceeded("GetFileInformationByHandleEx for FileIdInfo (" + p.size() + " bytes)",
+                        Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileIdInfo, p, new DWORD(p.size())));
+                FILE_ID_INFO fii = new FILE_ID_INFO(p);
+                // Volume serial number should be non-zero
+                assertFalse(fii.VolumeSerialNumber == 0);
             }
-            FILE_ID_INFO fii = new FILE_ID_INFO(p);
-            // Volume serial number should be non-zero
-            assertFalse(fii.VolumeSerialNumber == 0);
         } finally {
             Kernel32.INSTANCE.CloseHandle(hFile);
         }
@@ -1079,8 +1077,8 @@ public class Kernel32Test extends TestCase {
 
         try {
             Memory p = new Memory(FILE_BASIC_INFO.sizeOf());
-            if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileBasicInfo, p, new DWORD(p.size())))
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
+            AbstractWin32TestSupport.assertCallSucceeded("GetFileInformationByHandleEx for FileBasicInfo (" + p.size() + " bytes)",
+                    Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileBasicInfo, p, new DWORD(p.size())));
 
             FILE_BASIC_INFO fbi = new FILE_BASIC_INFO(p);
             // Add TEMP attribute
@@ -1091,11 +1089,11 @@ public class Kernel32Test extends TestCase {
             fbi.LastWriteTime = new WinNT.LARGE_INTEGER(0);
             fbi.write();
 
-            if (false == Kernel32.INSTANCE.SetFileInformationByHandle(hFile, WinBase.FileBasicInfo, fbi.getPointer(), new DWORD(FILE_BASIC_INFO.sizeOf())))
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
+            AbstractWin32TestSupport.assertCallSucceeded("SetFileInformationByHandle for FileBasicInfo (" + FILE_BASIC_INFO.sizeOf() + " bytes)",
+                    Kernel32.INSTANCE.SetFileInformationByHandle(hFile, WinBase.FileBasicInfo, fbi.getPointer(), new DWORD(FILE_BASIC_INFO.sizeOf())));
 
-            if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileBasicInfo, p, new DWORD(p.size())))
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
+            AbstractWin32TestSupport.assertCallSucceeded("GetFileInformationByHandleEx for FileBasicInfo (" + p.size() + " bytes)",
+                    Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileBasicInfo, p, new DWORD(p.size())));
 
             fbi = new FILE_BASIC_INFO(p);
             assertTrue((fbi.FileAttributes & WinNT.FILE_ATTRIBUTE_TEMPORARY) != 0);
@@ -1115,8 +1113,8 @@ public class Kernel32Test extends TestCase {
 
         try {
             FILE_DISPOSITION_INFO fdi = new FILE_DISPOSITION_INFO(true);
-            if (false == Kernel32.INSTANCE.SetFileInformationByHandle(hFile, WinBase.FileDispositionInfo, fdi.getPointer(), new DWORD(FILE_DISPOSITION_INFO.sizeOf())))
-                fail("SetFileInformationByHandle failed with " + Kernel32.INSTANCE.GetLastError());
+            AbstractWin32TestSupport.assertCallSucceeded("SetFileInformationByHandle for FileDispositionInfo (" + FILE_DISPOSITION_INFO.sizeOf() + " bytes)",
+                    Kernel32.INSTANCE.SetFileInformationByHandle(hFile, WinBase.FileDispositionInfo, fdi.getPointer(), new DWORD(FILE_DISPOSITION_INFO.sizeOf())));
 
         } finally {
             Kernel32.INSTANCE.CloseHandle(hFile);
@@ -1325,14 +1323,12 @@ public class Kernel32Test extends TestCase {
         Kernel32.INSTANCE.GetSystemTime(systemTime);
         WinBase.FILETIME fileTime = new WinBase.FILETIME();
 
-        if (false == Kernel32.INSTANCE.SystemTimeToFileTime(systemTime, fileTime)) {
-            fail("SystemTimeToFileTime failed with " + Kernel32.INSTANCE.GetLastError());
-        }
+        AbstractWin32TestSupport.assertCallSucceeded("SystemTimeToFileTime",
+                Kernel32.INSTANCE.SystemTimeToFileTime(systemTime, fileTime));
 
         WinBase.SYSTEMTIME newSystemTime = new WinBase.SYSTEMTIME();
-        if (false == Kernel32.INSTANCE.FileTimeToSystemTime(fileTime, newSystemTime)) {
-            fail("FileTimeToSystemTime failed with " + Kernel32.INSTANCE.GetLastError());
-        }
+        AbstractWin32TestSupport.assertCallSucceeded("FileTimeToSystemTime",
+                Kernel32.INSTANCE.FileTimeToSystemTime(fileTime, newSystemTime));
 
         assertEquals(systemTime.wYear, newSystemTime.wYear);
         assertEquals(systemTime.wDay, newSystemTime.wDay);
@@ -1359,9 +1355,8 @@ public class Kernel32Test extends TestCase {
         try {
 
             Memory p = new Memory(FILE_BASIC_INFO.sizeOf());
-            if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileBasicInfo, p, new DWORD(p.size()))) {
-                fail("GetFileInformationByHandleEx failed with " + Kernel32.INSTANCE.GetLastError());
-            }
+            AbstractWin32TestSupport.assertCallSucceeded("GetFileInformationByHandleEx for FileBasicInfo (" + p.size() + " bytes)",
+                    Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileBasicInfo, p, new DWORD(p.size())));
             FILE_BASIC_INFO fbi = new FILE_BASIC_INFO(p);
             FILETIME ft = new FILETIME(fbi.LastWriteTime);
             SYSTEMTIME stUTC = new SYSTEMTIME();
@@ -1389,13 +1384,31 @@ public class Kernel32Test extends TestCase {
 
     @Test
     public void testCreateRemoteThread() {
+        Assume.assumeTrue("testCreateRemoteThread is only implemented for x86 + x86-64", Platform.isIntel());
+
         Pointer addr = Kernel32.INSTANCE.VirtualAllocEx(
             Kernel32.INSTANCE.GetCurrentProcess(), null, new SIZE_T(4096),
             MEM_COMMIT | MEM_RESERVE, PAGE_EXECUTE_READWRITE);
 
-        // mov eax, ecx; ret; int3
         Memory localBuffer = new Memory(4096);
-        localBuffer.setInt(0, 0xccc3c18b);
+        localBuffer.clear();
+
+        if (Platform.is64Bit()) {
+            // mov eax, ecx; ret; int3
+            localBuffer.setByte(0, (byte) 0x8b); // MOV
+            localBuffer.setByte(1, (byte) 0xc1); // ECX -> EAX
+            localBuffer.setByte(2, (byte) 0xc3); // RET
+            localBuffer.setByte(3, (byte) 0xcc); // INT3
+        } else {
+            // mov eax, esp + 4; ret; int3
+            localBuffer.setByte(0, (byte) 0x8b); // MOV
+            localBuffer.setByte(1, (byte) 0x44); // ESP + 4bytes-> EAX
+            localBuffer.setByte(2, (byte) 0x24); //
+            localBuffer.setByte(3, (byte) 0x04); //
+            localBuffer.setByte(4, (byte) 0xc3); // RET
+            localBuffer.setByte(5, (byte) 0xcc); // INT3
+        }
+
         IntByReference bytesWritten = new IntByReference();
         Kernel32.INSTANCE.WriteProcessMemory(Kernel32.INSTANCE.GetCurrentProcess(),
             addr, localBuffer, 4096, bytesWritten);
@@ -1798,7 +1811,8 @@ public class Kernel32Test extends TestCase {
         // Set bit flags to 0x0001
         int previousMode = Kernel32.INSTANCE.SetErrorMode(0x0001);
         // Restore to previous state; 0x0001 is now "previous"
-        assertEquals(Kernel32.INSTANCE.SetErrorMode(previousMode), 0x0001);
+        // Since 0x0004 bit is sticky previous may be 0x0005
+        assertEquals(0x0001, Kernel32.INSTANCE.SetErrorMode(previousMode) & ~0x0004);
     }
 
 // Testcase is disabled, as kernel32 ordinal values are not stable.
@@ -1899,4 +1913,53 @@ public class Kernel32Test extends TestCase {
 
         assertNull(mutexHandle);
     }
+
+    public void testApplicationRestart() {
+        try {
+            HRESULT insufficientBuffer = W32Errors.HRESULT_FROM_WIN32(WinError.ERROR_INSUFFICIENT_BUFFER);
+            HRESULT result;
+            String dummyCommandline = "/restart -f .\\filename.ext";
+            char[] dummyCommandlineArray = Native.toCharArray(dummyCommandline);
+            int dummyFlags = 2;
+            result = Kernel32.INSTANCE.RegisterApplicationRestart(dummyCommandlineArray, dummyFlags);
+            assertTrue(COMUtils.SUCCEEDED(result));
+
+            char[] queriedCommandlineArray = null;
+            IntByReference queriedSize = new IntByReference();
+            IntByReference queriedFlags = new IntByReference();
+
+            // Query without target buffer to determine required buffer size
+            result = Kernel32.INSTANCE.GetApplicationRestartSettings(Kernel32.INSTANCE.GetCurrentProcess(), queriedCommandlineArray, queriedSize, queriedFlags);
+
+            assertTrue(COMUtils.SUCCEEDED(result));
+            assertEquals(dummyCommandline.length() + 1, queriedSize.getValue());
+
+            // Check error reporting, use insufficient buffer size
+            queriedCommandlineArray = new char[1];
+            queriedSize.setValue(queriedCommandlineArray.length);
+            queriedFlags.setValue(-1);
+            result = Kernel32.INSTANCE.GetApplicationRestartSettings(Kernel32.INSTANCE.GetCurrentProcess(), queriedCommandlineArray, queriedSize, queriedFlags);
+
+            assertTrue(COMUtils.FAILED(result));
+            assertEquals(insufficientBuffer, result);
+            assertEquals(dummyCommandline.length() + 1, queriedSize.getValue());
+
+            // Now query with the right buffer size
+            queriedCommandlineArray = new char[queriedSize.getValue()];
+            queriedSize.setValue(queriedCommandlineArray.length);
+            queriedFlags.setValue(-1);
+            result = Kernel32.INSTANCE.GetApplicationRestartSettings(Kernel32.INSTANCE.GetCurrentProcess(), queriedCommandlineArray, queriedSize, queriedFlags);
+
+            assertTrue(COMUtils.SUCCEEDED(result));
+            assertEquals(dummyCommandline.length() + 1, queriedSize.getValue());
+            assertEquals(dummyFlags, queriedFlags.getValue());
+            assertEquals(dummyCommandline, Native.toString(queriedCommandlineArray));
+
+            result = Kernel32.INSTANCE.UnregisterApplicationRestart();
+            assertTrue(COMUtils.SUCCEEDED(result));
+        } finally {
+            // Last resort if test succeeds partially
+            Kernel32.INSTANCE.UnregisterApplicationRestart();
+        }
+    }
 }
