@@ -96,6 +96,7 @@ public class SAFEARRAYTest {
     public void testCreateVarArray() {
         SAFEARRAY varArray = SAFEARRAY.createSafeArray(1);
         Assert.assertTrue(varArray != null);
+        varArray.destroy();
     }
 
     @Test
@@ -133,6 +134,7 @@ public class SAFEARRAYTest {
                 OleAuto.INSTANCE.VariantClear(element);
             }
         }
+        varArray.destroy();
     }
 
     @Ignore("Only for live testing")
@@ -198,11 +200,11 @@ public class SAFEARRAYTest {
         wrap.lock();
         int rowMax = wrap.getUBound(2);
         int columnMax = wrap.getUBound(1);
-        Object[][] result = new Object[(int) (rowMax + 1)][(int) (columnMax + 1)];
+        Object[][] result = new Object[rowMax + 1][columnMax + 1];
         for(int i = 0; i <= rowMax; i++) {
             for(int j = 0; j <= columnMax; j++) {
                 VARIANT cell = (VARIANT) wrap.getElement(i, j);
-                result[(int)i][(int) j] = cell.getValue();
+                result[i][j] = cell.getValue();
                 OleAuto.INSTANCE.VariantClear(cell);
             }
         }
@@ -214,11 +216,11 @@ public class SAFEARRAYTest {
         wrap.lock();
         int rowMax = wrap.getUBound(2);
         int columnMax = wrap.getUBound(1);
-        Object[][] result = new Object[(int) (rowMax + 1)][(int) (columnMax + 1)];
+        Object[][] result = new Object[rowMax + 1][columnMax + 1];
         for(int i = 0; i <= rowMax; i++) {
             for(int j = 0; j <= columnMax; j++) {
                 VARIANT cell = new VARIANT(wrap.ptrOfIndex(i, j));
-                result[(int)i][(int) j] = cell.getValue();
+                result[i][j] = cell.getValue();
             }
         }
         wrap.unlock();
@@ -266,7 +268,7 @@ public class SAFEARRAYTest {
         Assert.assertEquals(6, sa.getUBound(1));
 
         // requery the moved array and compare with basic array
-        Object[][] relocated = (Object[][]) OaIdlUtil.toPrimitiveArray(sa, false);
+        Object[][] relocated = (Object[][]) OaIdlUtil.toPrimitiveArray(sa, true);
         Assert.assertArrayEquals( basic, relocated);
     }
 
@@ -498,6 +500,7 @@ public class SAFEARRAYTest {
      * Test assumption: The windows search provider is present and holds at least
      * five entries. If this assumption is not met, this test fails.
      */
+    @Ignore("Assumes windows search provider present with five entries")
     @Test
     public void testADODB() {
         ObjectFactory fact = new ObjectFactory();
@@ -664,10 +667,13 @@ public class SAFEARRAYTest {
         SAFEARRAY arr = SAFEARRAY.createSafeArray(1);
         arr.putElement(new VARIANT("System.ItemUrl"), 0);
         boolean exceptionCaught = false;
+        VARIANT itemName = new VARIANT("System.ItemName");
         try {
-            arr.putElement(new VARIANT("System.ItemName"), 1);
+            arr.putElement(itemName, 1);
         } catch (COMException ex) {
             exceptionCaught = true;
+        } finally {
+            OleAuto.INSTANCE.VariantClear(itemName);
         }
         assertTrue("Array is initialized to a size of one - it can't hold a second item.", exceptionCaught);
         arr.redim(2, 0);
@@ -770,6 +776,7 @@ public class SAFEARRAYTest {
         }
         private long value;
 
+        @Override
         public long getValue() {
             return this.value;
         }
@@ -791,6 +798,7 @@ public class SAFEARRAYTest {
         }
         private long value;
 
+        @Override
         public long getValue() {
             return this.value;
         }
