@@ -1,23 +1,23 @@
 /* Copyright (c) 2007-2009 Timothy Wall, All Rights Reserved
  *
- * The contents of this file is dual-licensed under 2 
- * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * The contents of this file is dual-licensed under 2
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and
  * Apache License 2.0. (starting with JNA version 4.0.0).
- * 
- * You can freely decide which license you want to apply to 
+ *
+ * You can freely decide which license you want to apply to
  * the project.
- * 
+ *
  * You may obtain a copy of the LGPL License at:
- * 
+ *
  * http://www.gnu.org/licenses/licenses.html
- * 
+ *
  * A copy is also included in the downloadable source code package
  * containing JNA, in file "LGPL2.1".
- * 
+ *
  * You may obtain a copy of the Apache License at:
- * 
+ *
  * http://www.apache.org/licenses/
- * 
+ *
  * A copy is also included in the downloadable source code package
  * containing JNA, in file "AL2.0".
  */
@@ -37,15 +37,15 @@ import junit.framework.TestCase;
 /** Test loading and unloading native support from various locations.  Note
  * that no JNI classes are directly referenced in these tests.
  */
-public class JNALoadTest extends TestCase implements Paths, GCWaits {
+public class JNALoadTest extends TestCase implements Paths {
 
     private class TestLoader extends URLClassLoader {
         public TestLoader(boolean fromJar) throws MalformedURLException {
-            super(new URL[] {
-                    Platform.isWindowsCE()
-                    ? new File("/Storage Card/" + (fromJar ? "jna.jar" : "test.jar")).toURI().toURL()
-                    : new File(BUILDDIR + (fromJar ? "/jna.jar" : "/classes")).toURI().toURL(),
-                  }, new CloverLoader());
+            super(new URL[]{
+                Platform.isWindowsCE()
+                ? new File("/Storage Card/" + (fromJar ? "jna.jar" : "test.jar")).toURI().toURL()
+                    : new File(BUILDDIR + (fromJar ? "/jna.jar" : "/classes")).toURI().toURL()},
+                new CloverLoader());
             if (fromJar) {
                 assertJarExists();
             }
@@ -125,7 +125,7 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         String path = (String)field.get(null);
         assertNotNull("Native library path unavailable", path);
         assertTrue("Native library not unpacked from jar: " + path,
-                   path.startsWith(System.getProperty("java.io.tmpdir")));
+                path.startsWith(Native.getTempDir().getAbsolutePath()));
 
         Reference<Class<?>> ref = new WeakReference<Class<?>>(cls);
         Reference<ClassLoader> clref = new WeakReference<ClassLoader>(loader);
@@ -133,19 +133,16 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         cls = null;
         field = null;
         System.gc();
-        for (int i=0;i < GC_WAITS && (ref.get() != null || clref.get() != null);i++) {
-            Thread.sleep(GC_WAIT_INTERVAL);
-            System.gc();
+        for (int i=0;i < GCWaits.GC_WAITS && (ref.get() != null || clref.get() != null);i++) {
+            GCWaits.gcRun();
         }
         assertNull("Class not GC'd: " + ref.get(), ref.get());
         assertNull("ClassLoader not GC'd: " + clref.get(), clref.get());
 
         // Check for temporary file deletion
         File f = new File(path);
-        for (int i=0;i < GC_WAITS && (f.exists() || Boolean.getBoolean("jna.loaded"));i++) {
-            System.gc(); // attempt to fix intermittent test failures
-            Thread.sleep(4 * GC_WAIT_INTERVAL);  // '4 *' is attempt to fix intermittent test failures
-            System.gc();
+        for (int i=0;i < GCWaits.GC_WAITS && (f.exists() || Boolean.getBoolean("jna.loaded"));i++) {
+            GCWaits.gcRun();
         }
 
         if (f.exists()) {
@@ -186,9 +183,8 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         cls = null;
         field = null;
         System.gc();
-        for (int i=0;i < GC_WAITS && (ref.get() != null || clref.get() != null || Boolean.getBoolean("jna.loaded"));i++) {
-            Thread.sleep(2 * GC_WAIT_INTERVAL);  // '2 *' is attempt to fix intermittent test failures 
-            System.gc();
+        for (int i=0;i < GCWaits.GC_WAITS && (ref.get() != null || clref.get() != null || Boolean.getBoolean("jna.loaded"));i++) {
+            GCWaits.gcRun();
         }
         assertNull("Class not GC'd: " + ref.get(), ref.get());
         assertNull("ClassLoader not GC'd: " + clref.get(), clref.get());
@@ -197,9 +193,8 @@ public class JNALoadTest extends TestCase implements Paths, GCWaits {
         Throwable throwable = null;
         // NOTE: IBM J9 needs some extra time to unload the native library,
         // so try a few times before failing
-        for (int i=0;i < GC_WAITS;i++) {
-            System.gc();
-            Thread.sleep(GC_WAIT_INTERVAL);
+        for (int i=0;i < GCWaits.GC_WAITS;i++) {
+            GCWaits.gcRun();
             try {
                 loader = new TestLoader(false);
                 cls = Class.forName("com.sun.jna.Native", true, loader);
